#
# gui.py
#
# Python Digital Audio Recorder - GUI module
#
# Copyright (C) 2003 Daniel Thompson <d.thompson@gmx.net> 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#

import os, os.path, pickle, string, sys, time

import pygtk
pygtk.require("2.0")
import gobject, gtk, gtk.glade

class PydarGui:
	""" Class to control the main pydar window """

	def __init__(self, interface, state, tuner):
		self.tuner = tuner
		self.stateFile = state

		# open the widget tree and initialize it with this class
		self.widgetTree = gtk.glade.XML(interface)
		self.widgetTree.signal_autoconnect(self)

		# set the headings for the main list
		self.listModel = gtk.ListStore(
			gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING,
			gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_PYOBJECT
		)
		treeView = self.get_widget("main_treeview1")
		treeView.set_model(self.listModel)
		i = 0
		for heading in "Station", "Date", "Start time", "End time", "Filename":
			renderer = gtk.CellRendererText()
			column = gtk.TreeViewColumn(heading, renderer, text=i)
			column.set_resizable(gtk.TRUE)
			treeView.append_column(column)

			# generate a constant to reference this column
			self.__dict__[heading.replace(' ', '_').upper()] = i

			i += 1
		self.ID = i
			
		# create a dictionary to contain the programmes
		try:
			self.load_state()
		except IOError:
			self.programmes = {}
		except EOFError:
			# TODO: GUI error alert (corrupt state file)
			print "Pydar's statefile is corrupt, starting from scratch"
			self.programmes = {}
			

		# cache some of the most frequently looked up names
		self.programme_spinbuttons = (
			self.get_widget("programme_spinbutton1"),
			self.get_widget("programme_spinbutton2"),
			self.get_widget("programme_spinbutton3"),
			self.get_widget("programme_spinbutton4"),
		)

		# create a class instance to manage the preferences
		self.preferences = PydarPreferences(self.widgetTree)

	def update_model(self, iter, id, programme):
		lm = self.listModel
		station, (day, month, year), start, end, fname = programme
		lm.set_value(iter, self.STATION,    station)
		lm.set_value(iter, self.DATE,       "%2d/%2d/%4d" % (day, month, year))
		lm.set_value(iter, self.START_TIME, "%2d:%02d" % start)
		lm.set_value(iter, self.END_TIME,   "%2d:%02d" % end)
		lm.set_value(iter, self.FILENAME,   fname)
		lm.set_value(iter, self.ID,         id)
		
	def add_programme(self, id, programme):
		lm = self.listModel
		i = lm.append()
		self.update_model(i, id, programme)

	def load_state(self):
		f = open(self.stateFile, 'r')
		self.programmes = pickle.load(f)
		f.close()

		# TODO: remove expired items from the loaded dictionary
		list = self.tuner.get_id_list()
		
		# register each of the items in the GUI
		for id, programme in self.programmes.items():
			if id in list:
				self.add_programme(id, programme)
			else:
				del self.programmes[id]

		self.save_state()

	def save_state(self):
		# TODO: this is not safe if the disk is full!
		f = open(self.stateFile, 'w')
		pickle.dump(self.programmes, f)
		f.close()

	def get_widget(self, name):
		""" short hand to obtain named widgets """
		return self.widgetTree.get_widget(name)

	def on_about_open(self, widget):
		""" display and manage the about dialog box """
		d = self.get_widget("about")
		d.run()
		d.hide()
		
	def on_quit1_activate(self, widget):
		""" handle user request to close the application """
		gtk.mainquit()

	def on_main_delete_event(self, widget, event):
		""" handle window manager request to close the applciation """
		gtk.mainquit()

	def on_new_programme_open(self, widget):
		# initalize the various widgets
		timespec = time.localtime(time.time())
		(hh, mm, ehh, emm) = self.programme_spinbuttons
		hh.set_value(timespec[3])
		mm.set_value(timespec[4])
		# TODO: this is not 'midnight safe'
		ehh.set_value(timespec[3]+1)
		emm.set_value(timespec[4])

		filename = self.get_widget("programme_entry1")
		filename.set_text("")
		self.on_programme_filename_changed(filename)

		# run the dialog
		programme = self.on_programme_open(widget, "New Programme")
		if None == programme:
			return

		# get the tuner backend to record the program
		id = self.tuner.add_programme(programme)

		# add it to the GUI
		self.add_programme(id, programme)

		# record (and preserve) our state
		self.programmes[id] = programme
		self.save_state()

	def do_modify_selection(self, listModel, index, iter):
		id = listModel.get_value(iter, self.ID)
		station, (day, month, year), (h, m), (eh, em), fname = self.programmes[id]

		calendar = self.get_widget("programme_calendar1")
		calendar.select_month(month-1, year)
		calendar.select_day(day)

		(hh, mm, ehh, emm) = self.programme_spinbuttons
		hh.set_value(h)
		mm.set_value(m)
		ehh.set_value(eh)
		emm.set_value(em)

		filename = self.get_widget("programme_entry1")
		filename.set_text(fname)
		self.on_programme_filename_changed(filename)
	
		# run the dialog
		programme = self.on_programme_open(listModel, "Modify Programme", station)
		if None == programme:
			return

		# delete the existing entry
		self.tuner.delete_programme(id)
		del self.programmes[id]

		# recreate a new entry
		id = self.tuner.add_programme(programme)
		self.update_model(iter, id, programme)
		self.programmes[id] = programme
		self.save_state()

	def on_modify_programme_open(self, widget):
		tv = self.get_widget("main_treeview1")
		selection = tv.get_selection()
		selection.selected_foreach(self.do_modify_selection)
		

	def on_programme_open(self, widget, title, select=None):
		""" display and manage the programme dialog box """
		d = self.get_widget("programme")
		d.set_title(title)
		om = self.get_widget("programme_optionmenu1")
		filename = self.get_widget("programme_entry1")

		# construct the station menu
		menu = gtk.Menu()
		stations = self.tuner.get_stations()
		i = 0
		for s in stations:
			mi = gtk.MenuItem(s)
			mi.show()
			menu.append(mi)
			if select == s:
				menu.set_active(i)
			i += 1
		om.set_menu(menu)

		# preform the dialog
		response = d.run()
		d.hide()

		# ignore the results if we were not dismissed by an OK
		if response != gtk.RESPONSE_OK:
			return

		# extract the values from the dialog
		station = om.get_children()[0].get()
		(year, month, day) = self.get_widget("programme_calendar1").get_date()
		month += 1		# in GTK month is an enumeration and starts at 0!
		start = (self.programme_spinbuttons[0].get_value_as_int(),
		         self.programme_spinbuttons[1].get_value_as_int())
		end   = (self.programme_spinbuttons[2].get_value_as_int(),
		         self.programme_spinbuttons[3].get_value_as_int())
		# get filename and make it absolute
		fname =  os.path.abspath(filename.get_text())

		# pack as a tuple and return the recording
		return (station, (day, month, year), start, end, fname)

	def on_programme_filename_changed(self, widget):
		""" validate the filename and grey the OK button if it is wrong """
		ok = self.get_widget("programme_okbutton1")
		if widget.get_text() == "":
			ok.set_sensitive(gtk.FALSE)
		else:
			ok.set_sensitive(gtk.TRUE)

	def on_programme_fileselector_open(self, widget):
		""" display a fileselector and apply the results to the programme dialog """
		entry = self.get_widget("programme_entry1")
		self.do_fileselector_for_entry(entry)

	def do_delete_selection(self, listModel, index, iter):
		# get the id of the programme to delete
		id = listModel.get_value(iter, self.ID)

		# we must remove if from the tuner first (if it fails it will generate an
		# exception)
		self.tuner.delete_programme(id)

		# now remove it from the GUI and the stored state
		listModel.remove(iter)
		del self.programmes[id]
		self.save_state()

	def on_delete_programme(self, widget):
		""" remove recorded programmes from the list (and the tuner) """

		# TODO: are you sure?

		tv = self.get_widget("main_treeview1")
		selection = tv.get_selection()
		selection.selected_foreach(self.do_delete_selection)
	
	def do_fileselector_for_entry(self, entry):
		fs = gtk.FileSelection()
		fs.set_filename(entry.get_text())
		response = fs.run()
		if response == gtk.RESPONSE_OK:
			entry.set_text(fs.get_filename())
		fs.destroy()

class PydarPreferences:
	""" Class to manage the Pydar prefences dialog """

	def __init__(self, widgetTree):
		self.widgetTree = widgetTree
		self.widgetTree.signal_autoconnect(self)

	# TODO: this should be factored out into a common super-class
	def get_widget(self, name):
		""" short hand to obtain named widgets """
		return self.widgetTree.get_widget(name)

	def on_preferences1_activate(self, widget):
		d = self.get_widget('preferences')
		d.run()
		d.hide()
		
