
# tuner.py
#
# Python Digital Audio Recorder
#
# Copyright (C) 2003 Daniel Thompson <d.thompson@gmx.net> 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#

import os, string, sys, videodev

class Tuner:
	""" class to interact with the specific tuner backend """

	stations = {
		'BBC Radio 1' : 99.5,
		'BBC Radio 4' : 94.3,
		'Classic FM'  : 101.70,
	}

	def __init__(self, scriptFile):
		self.scriptFile = scriptFile

		# speculatively extract gnomeradio's presets
		try:
			self.query_gnomeradio()
		except ImportError:
			# No gconf
			pass
		except NameError:
			# No gnomeradio
			pass

	def query_gnomeradio(self):
		""" use gconf to extract gnomeradio's station presets """
		import gconf

		client = gconf.client_get_default()
		key = '/apps/gnomeradio/presets'

		# register which directory we are interested in
		client.add_dir(key, gconf.CLIENT_PRELOAD_NONE)

		# TODO: no notifications
			
		presets = client.get_int(key + '/presets')
		if 0 == presets:
			raise NameError, 'gconf key does not exist'

		self.stations = {}
		for i in range(presets):
			name = client.get_string(key + '/%d/name' % i)
			freq = client.get_float(key + '/%d/freqency' % i)
			self.stations[name] = freq

	def get_stations(self):
		s = self.stations.keys()
		s.sort()
		return s

	def get_id_list(self):
		""" generate a list of valid ids to allow the list to be pruned """
		cout = os.popen("atq", 'r')
		jobs = cout.readlines()
		cout.close()

		list = []

		for job in jobs:
			id = string.split(job)[0]
			list.append(string.atoi(id))

		return list

	def add_programme(self, programme):
		""" generate a job to cause the programme to be recorded """

		# unpack the tuple argument
		station, (dd, mm, yy), (HH, MM), (eHH, eMM), fname = programme

		# calculate the number of seconds that has been requested.
		if eHH < HH or (eHH == HH and eMM <= MM): eHH += 24
		seconds = (((eHH * 60) + eMM) * 60) - (((HH * 60) + MM) * 60)

		cmd = "at %02d:%02d %02d.%02d.%02d" % (HH, MM, dd, mm, yy)
		cin, cout = os.popen4(cmd)

		# record at CD quality with two channels
		rec = 'sox -tossdsp -r44100 -sw -c2 /dev/dsp'

		# add any compression that the file extender requests
		if fname.endswith('.ogg'):
			rec += ' -twav - 2> /dev/null | oggenc - --quiet --output='
		elif fname.endswith('.spx'):
			# to compress with speex we will downsample to one of its optimized
			# sample rates before encoding
			rec += ' -twav -r16000 - resample 2> /dev/null | speexenc - '
		else:
			# any unknown extension will be stored in Microsoft WAV format
			rec += ' -twav '

		rec += fname
		#print rec

		# generate a shell script that:
		#  - tunes to the requested radio station
		#  - ?sets up the mixer?
		#  - starts recording
		#  - sleeps for some period of time
		#  - stops recording and exits
		cin.write("""
# configure the hardware to do what needs to be done
pydar=%s
$pydar --tune="%s"
#echo TODO: set the mixer to the radio source

%s > /dev/null 2>&1 &
pid=$!

# wait for the recording to finish
sleep %d

# stop the recording
$pydar --mute
kill $!
"""			% (self.scriptFile, station, rec, seconds))
		cin.close()
		out = string.split(cout.readline())
		cout.close()

		# return the second word (the job id) as an integer
		return string.atoi(out[1])

	def delete_programme(self, id):
		""" remove the specified job """
		os.system("atrm %d" % id)

	def tune(self, channel):
		vd = videodev.VideoDev('/dev/radio')

		# tune to the requested station
		freq = (self.stations[channel] + (1.0/32.0)) * 16
		if 'VIDEO_TUNER_LOW' in vd.get_tuner()['flags']:
			freq = freq * 1000
		vd.set_freq(freq)
		
		# unmute the device
		aud = vd.get_audio()
		aud['volume'] = 65535
		if 'VIDEO_AUDIO_MUTE' in aud['flags']:
			aud['flags'].remove('VIDEO_AUDIO_MUTE')
		vd.set_audio(aud)

		vd.close()
	
	def mute(self):
		vd = videodev.VideoDev('/dev/radio')

		# set volume to 0 and mute
		aud = vd.get_audio();
		aud['volume'] = 0
		aud['flags'].append('VIDEO_AUDIO_MUTE')
		vd.set_audio(aud)

		# tune to nothing
		vd.set_freq(0)

		vd.close()
